#!/bin/bash

sox --version > /dev/null 2>&1
if [ $? -ne 0 ]; then
   echo "Please install sox!";
   exit 1;
fi
 
if [ $# -lt 2 ] || [ $# -gt 3 ]; then
   echo "$0: Fold an audio file on itself using sox."
   echo "Usage: There are 2 modes of operation"
   echo "-------------------------------------"
   echo "[1] INPUTFILE   TARGET_DURATION"
   echo "TARGET_DURATION may be <seconds.milliseconds> or <samples>s"
   echo "e.g. $0 abc.wav 4.58"
   echo "to fold at 4 seconds 58 milliseconds"
   echo ""
   echo "e.g. $0 abc.wav 4895832s"
   echo "to fold at sample 4895832. Note the s after the sample number."
   echo ""
   echo "[2] INPUTFILE   BEATS_PER_MINUTE   NUMBER_OF_BEATS"
   echo "e.g. $0 abc.wav 130 8"
   echo "to fold at 8 beats, given that the input file has 130 beats/minute"
   echo ""
   exit 1;
fi
name=$1

if [ $# -eq 3 ]; then
   srate=$(soxi -r $1);
   if [ -n "$srate" ]; then
      dur=$(echo "" | awk '{a = sprintf("%.2f\n", srate*beats*60/bpm); if(a ~ /\.[56789]/){a = int(a); a++}else{a = int(a)} print a}' srate=$srate bpm=$2 beats=$3)s;
   else
      echo "Could not get samplerate of $name";
      exit 1;
   fi
else
   dur=$2
fi

echo "DURATION: $dur";

name_start=$(echo $name|awk -F'.' '{print $1}')
name_end=$(echo $name|awk -F'.' '{print $2}')

if echo $dur | grep -q '^[0-9]\+s$'; then
   insize=$(soxi -s $name 2> /dev/null);
   dur=$(echo $dur | sed -e 's/s$//');
   suffix='s'
elif echo $dur | grep -q '^[0-9]\+\.\?[0-9]\+\?$'; then
   insize=$(soxi -D $name 2> /dev/null);
fi

if [ -z "$insize" ]; then
   echo "No valid audio input file. Could not determine length. Nothing to do.";
   exit 1;
fi
echo $dur $insize | awk '{if($1 >= $2 || $1 <= 0){ exit 1}}';
if [ $? -ne 0 ]; then
   echo "$dur >= size of input file: $insize. Or $dur <= 0. Nothing to do."; 
   exit 2;
fi
timez=$(echo $dur $insize | awk '{print int($2/$1)}');
c=0;
erg="";
while [ $c -le $timez ]; do
   a=$(echo $dur $c | awk '{print $1*$2}');
   b=$dur;
   if [ $c -eq $timez ]; then
      b='-0'
   fi
   sox $name /tmp/${name_start}_$c.$name_end trim $a$suffix $b$suffix;
   erg=$erg" -v 1.0 /tmp/${name_start}_$c.$name_end"; 
   let c++;
done
if [ -n "$erg" ]; then
   sox -m $erg /tmp/${name_start}_out.$name_end;
   rm $(echo $erg | sed -e 's/-v 1.0//g');

   echo "Output is /tmp/${name_start}_out.$name_end"
   printf "Duration: "; soxi -D /tmp/${name_start}_out.$name_end;
   printf "#Samples: "; soxi -s /tmp/${name_start}_out.$name_end;
fi
